﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Data
{
    /// <summary>
    /// Datová třída reprezentující fakturu
    /// </summary>
    public class Invoice
    {
        private int id;
        private String name;
        //odběratel
        private Person subscriber;
        //dodavatel
        private Person supplier;
        private List<Item> items = new List<Item>();
        //datum vystavení
        private DateTime created;
        //datum splatnosti
        private DateTime dueDate;
        //počet dní splatnosti
        private int dueDays;
        private bool paid;


        //konstruktor
        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="id">int, ID faktury</param>
        public Invoice(int id)
        {
            this.id = id;
            this.created = DateTime.Today;
        }

        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="id">int, ID faktury</param>
        /// <param name="name">string, typ faktury</param>
        /// <param name="subscriber">Person, odběratel</param>
        /// <param name="supplier">Person, dodavatel</param>
        /// <param name="dueDate">int, počet dní splatnosti</param>
        public Invoice(int id, String name, Person subscriber, Person supplier, int dueDate)
            : this(id, name, subscriber, supplier, dueDate, false)
        {
        }

        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="id">int, ID faktury</param>
        /// <param name="name">string, typ faktury</param>
        /// <param name="subscriber">Person, odběratel</param>
        /// <param name="supplier">Person, dodavatel</param>
        /// <param name="dueDays">int, počet dní splatnosti</param>
        /// <param name="paid">bool, TRUE je-li faktura již zaplacená</param>
        public Invoice(int id, String name, Person subscriber, Person supplier, int dueDate, bool paid)
            : this(id, name, subscriber, supplier, dueDate, false, DateTime.Today)
        {
        }

        /// <summary>
        /// Kostruktor - úplný
        /// </summary>
        /// <param name="id">int, ID faktury</param>
        /// <param name="name">string, typ faktury</param>
        /// <param name="subscriber">Person, odběratel</param>
        /// <param name="supplier">Person, dodavatel</param>
        /// <param name="dueDays">int, počet dní splatnosti</param>
        /// <param name="paid">bool, TRUE je-li faktura již zaplacená</param>
        /// <param name="created">DateTime, datum vytvoření (vystavení) faktury</param>
        public Invoice(int id, String name, Person subscriber, Person supplier, int dueDays, bool paid, DateTime created)
        {
            this.id = id;
            this.name = name;
            this.subscriber = subscriber;
            this.supplier = supplier;
            this.created = created;
            this.dueDate = created;
            this.dueDate = this.dueDate.AddDays(dueDays);
            this.dueDays = dueDays;
            this.paid = paid;
        }
       
        //práce s datem a časem
        //DateTime date = DateTime.Today;           //dnešek
        //date = date.AddDays(7);                   //k nastavenemu datu přičte 7 dní

        //getry a setry
        /// <summary>
        /// Jedinečné ID faktruy
        /// </summary>
        public int ID
        {
            get { return id; }
            set { id = value; }
        }

        /// <summary>
        /// Typ položky - Vydaná/Přijatá
        /// </summary>
        public String Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <value>
        /// Odběratel
        /// </value>
        public Person Subscriber
        {
            get { return subscriber; }
            set { subscriber = value; }
        }

        /// <value>
        /// Dodavatel
        /// </value>
        public Person Supplier
        {
            get { return supplier; }
            set { supplier = value; }
        }

        /// <summary>
        /// Seznam položek
        /// </summary>
        public List<Item> Items
        {
            get { return items; }
        }

        /// <summary>
        /// Datum vytvoření
        /// </summary>
        public DateTime Created
        {
            get { return created; }
            set { created = value; }
        }

        /// <summary>
        /// Datum splatnosti
        /// </summary>
        public DateTime DueDate
        {
            get { return dueDate; }
        }

        /// <summary>
        /// Počet dní do splatnosti ode dne vytvoření. Setter nastaví i datum splatnosti
        /// </summary>
        public int DueDays
        {
            get { return dueDays; }
            set
            {
                dueDays = value;
                //upraví i datum splatnosti
                dueDate = created;
                dueDate.AddDays(value);
            }
        }

        /// <summary>
        /// Označení pro zaplacené faktury
        /// </summary>
        public bool Paid
        {
            get { return paid; }
            set { paid = value; }
        }


        //metody

        ///<sumary>
        /// Metoda přidá položku do seznamu
        /// @param Item item - jedna položka
        ///</sumary>
        public void addItem(Item item)
        {
            this.items.Add(item);
        }

        ///<sumary>
        /// Metoda spočítá a vrátí celkovou hodnotu faktury 
        /// @return int
        ///</sumary>
        public int totalPrice()
        {
            int totalSum = 0;
            foreach (Item item in items)
            {
                totalSum += item.Price;
            }
            return totalSum;
        }

        /// <summary>
        /// Informace o faktuře pro uložení do souboru
        /// </summary>
        /// <returns>string</returns>
        public string toSave()
        {
            return "" + id + "|" + name + "|" + subscriber.CompanyId + "|" + supplier.CompanyId + "|" + dueDays + "|" + paid + "|" + created;
        }

        /// <summary>
        /// Informace o faktuře pro výpis
        /// </summary>
        /// <returns>string</returns>
        public override string ToString()
        {
            return "Invoice: " + id + " " + name + " splatnost: " + dueDate
                    + " položek " + items.Count + " za " + totalPrice() + "Kč je " + (paid == true ? "zaplacená" : "nezaplacená");
        }

        /// <summary>
        /// Porovná dva objekty, jestli jsou stejné
        /// </summary>
        /// <param name="obj">object</param>
        /// <returns>TRUE objekty jsou stejné, jinak FALSE</returns>
        public override bool Equals(object obj)
        {
            if (this.GetHashCode() == obj.GetHashCode())
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Hashcode pro porovnávání
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return this.id;
        }


    }
}
